#include "main.h"

void uci_send (const char format[], ...) // Fruit
{
    va_list  arg_list;
    char     str [4096];

    va_start (arg_list, format);
    vsprintf (str, format, arg_list);
    va_end   (arg_list);

    /*
    if (strlen(str) > 0)
    {
       fprintf (f_out, "%s", str); //     GUI  
    };
    */

    fprintf  (stdout, "%s", str);
    fflush   (stdout);
};

void uci_wait ()
{
    char * ptr;
	char s[2048] = "";

	for (;;)
	{
		fgets (s, 2040, stdin);
		if (feof(stdin)) continue;

		ptr = strchr (s, '\n'); //    
        if (ptr != NULL) *ptr = '\0';

		if (strlen (s) > 0) fprintf (f_out, "[[%s", s);
		if (read_command (s, "quit") != NULL)
		{
			fclose (f_out);
			exit (0);
		};
		if (read_command (s, "uci") != NULL)
		{
			uci_send ("id name %s v%s\n", NAME, VERSION);
			uci_send ("id author %s\n", AUTHOR);

			uci_send ("option name Hash type spin default 1 min 1 max 128\n");

			uci_send ("uciok\n");
			break;
		};
		if (read_command (s, ".") != NULL)
		{
			//debug = 1;
			//break;
		};
	};
};

int uci_parsing (board * B)
{
    //     (  )
    //static int first_parse = 1;

    static int ready_to_search = 1;

    char * ptr;

	int j, k;
	//int score;
	U64 perft;
	int wtime, btime;//, inc;
	U64 x64;

    char s[2048] = {0, };

	char *d, *f;

	while (S->state != STATE_EXIT)
	{
	    if (!input_available()) break;

        fgets (s, 2040, stdin);

        ptr = strchr (s, '\n'); //   
        if (ptr != NULL) *ptr = '\0';

        //if (feof (stdin)) break;

        if (strlen(s) > 0) fprintf (f_out, "[[%s\n", s);

        if (read_command (s, "quit") != NULL)
        {
            fclose (f_out);
            S->state = STATE_EXIT;
            return 1;
        };

        if (read_command (s, "isready") != NULL)
        {
            uci_send ("readyok\n");
            break;
        };

        if ((d = read_command (s, "position ")) != NULL)
        {
            if ((f = read_command (d, "fen ")) != NULL)
            {
                fen_to_pos (f, B);

                if ((d = read_command (f, "moves ")) != NULL)
                {
                    moves_to_pos (d, B);
                };
                break;
            };

            if ((f = read_command (d, "startpos")) != NULL)
            {
                fen_to_pos (START_FEN, B);

                //print_board (L);

                if ((d = read_command (f, " moves ")) != NULL)
                {
                    moves_to_pos (d, B);
                };
                break;
            };
        };

        if ((d = read_command (s, "go")) != NULL)
        {
            if (!ready_to_search) break;

            wtime = btime = 100000;

            hash_refresh ();
            search_init ();

            ready_to_search = 0; //   -        

            S->state = ANALYSING;

            if ((read_command (d, "infinite")) != NULL)
            {
                S->infinite = 1;
            };

            //    wtime  btime !
            if ((f = read_command (d, "wtime ")) != NULL)
            {
                wtime = read_number (f);

                if ((f = read_command (d, "btime ")) != NULL)
                {
                    btime = read_number (f);
                };
            };

            S->critl_time = S->elaps_time = wtm ? wtime : btime;

            if (S->elaps_time)
            {
                S->elaps_time /= 50; //   (/50)
                S->critl_time /= 10; //  e (/10)
            };

            S->is_critical = 0;

            iid (GO_MAX_DEPTH, B);

            ready_to_search = 1; //     -    

            hash_refresh ();

            break;
        };

        if ((d = read_command (s, "setoption name ")) != NULL)
        {
            if ((f = read_command (d, "Hash ")) != NULL)
            {
                if ((d = read_command (f, "value ")) != NULL)
                {
                    j = read_number (d);

                    if (j > 0 && j <= 512) //    512  
                    {
                        delete H;
                        H = new hash(j * (1 << 20));
                    }
                    else uci_send ("info string I can't allocate more than 512Mb memory for hash\n");
                };
            };

            break;
        };

        if (read_command (s, "bench") != NULL)
        {
            for (j = BENCH_LW_DEPTH; j <= BENCH_HG_DEPTH; j ++)
            {
                perft = perft_root (-inf, inf, j, B);
            };

            break;
        };

        if ((d = read_command (s, "perft")) != NULL)
        {
            j = 1;

            if ((f = read_command (d, " ")) != NULL) if (k = read_number (f)) j = k;

            perft_root (-inf, inf, j, B);

            break;
        };

        if (read_command (s, "print") != NULL)
        {
            print_board (B);

            break;
        };

        if (read_command (s, "score") != NULL)
        {
            print_board (B);
            uci_send (" Score: %i\n\n", evaluate (B));

            break;
        };

        if (read_command (s, "stop") != NULL)
        {
            S->state = WAITING;
            print_score (S->score);
            uci_send ("pv %s\n", move_to_str (S->best_move));
            uci_send ("bestmove %s \n", move_to_str (S->best_move));

            break;
        };

        if (read_command (s, "help") != NULL)
        {
            uci_send ("\n");
            uci_send (" setoption name [NAME] value [VALUE]             - set option state (UCI)\n");
            uci_send (" position (fen [FEN] (moves [MOVES])) (startpos) - set new position (UCI)\n");
            uci_send (" go (wtime [TIME]) (btime [TIME])                - start to analyse (UCI)\n");
            uci_send (" isready                                         - don't sleep!     (UCI)\n");
            uci_send (" quit                                            - exit             (UCI)\n\n");
            uci_send (" perft ([DEPTH])                                 - performance test \n");
            uci_send (" bench                                           - benchmark based on perft\n");
            uci_send (" print                                           - print out position\n");
            uci_send (" score                                           - position score\n");
            uci_send (" .                                               - print out fen\n\n");

            break;
        };

        if (read_command (s, ".") != NULL)
        {
            uci_send ("%s to move\n", wtm ? "White" : "Black");
            uci_send ("fen: %s\n", pos_to_fen (B));

            break;
        };
	};

	return 0;
};


void uci_info ()
{
    double  nps; //    
    clock_t curr, diff;
    char    pv_line [MAX_DEPTH * 10] = {0, };
    double  diff_tm;

    if (S->state != ANALYSING) return;

    curr = clock ();
    diff = curr - S->curr_time;

    //      ,  -  ,   
    if (!S->infinite
    &&  S->best_move
    &&  S->search_depth >= DEPTH_TO_MOVE
    &&  curr - S->think_time > S->elaps_time)
    {
        if (S->is_critical)
        {
            if (curr - S->think_time > S->critl_time)
            {
                print_score (S->score);
                uci_send ("pv %s\n", move_to_str (S->best_move));
                uci_send ("bestmove %s \n", move_to_str (S->best_move));
                S->state = WAITING;
                return;
            };
        }
        else
        {
            print_score (S->score);
            uci_send ("pv %s\n", move_to_str (S->best_move));
            uci_send ("bestmove %s \n", move_to_str (S->best_move));
            S->state = WAITING;
            return;
        };
    };

    if (S->nodes_count > NODES_OF_OUTPUT)
    {
        diff_tm = (double)diff / CLOCKS_PER_SEC;

        if (!diff) nps = 0;
        else       nps = S->nodes_count / diff_tm;

        // info time 42219 nodes 8490000 nps 201094 cpuload 1000
        uci_send ("info time %i ", (int)(curr - S->think_time));
        uci_send ("nodes %i ",  S->nodes_count);
        uci_send ("nps %li ", (long)nps);
        uci_send ("cpuload 1000\n");

        S->curr_time = curr;
        S->nodes_count = 0;
    };

    if (S->pv_updated && S->score_last != S->score && S->search_depth >= DEPTH_TO_MOVE)
    {
        S->score_last = S->score;

        sprint_pv (S->search_depth, pv_line);

        print_score (S->score);

        uci_send ("pv %s\n", pv_line);

        S->pv_updated = 0;
    };
};
